import { PTransform } from "./transform";
import { PCollection, Root } from "../pvalue";
import { Coder } from "../coders/coders";
import { KV } from "../values";
import { CombineFn } from "./group_and_combine";
/**
 * `Impulse` is the basic *source* primitive `PTransformClass`. It receives a Beam
 * Root as input, and returns a `PCollection` of `Uint8Array` with a single
 * element with length=0 (i.e. the empty byte array: `new Uint8Array("")`).
 *
 * `Impulse` is used to start the execution of a pipeline with a single element
 * that can trigger execution of a source or SDF.
 */
export declare function impulse(): PTransform<Root, PCollection<Uint8Array>>;
export declare namespace impulse {
    var urn: string;
}
export declare function withCoderInternal<T>(coder: Coder<T>): PTransform<PCollection<T>, PCollection<T>>;
/**
 * Returns a PTransform returning a PCollection with the same contents as the
 * input PCollection, but with the given Row (aka Schema'd) Coder, based on
 * an "example" element. E.g.
 *
 *```js
 * const unknown_schema_pcoll = ...
 * const schema_pcoll = unknown_schema_pcoll.apply(
 *     withRowCoder({int_field: 0, str_field: ""}));
 *```
 *
 * This is particularly useful for declaring the type of the PCollection to
 * invoke cross-language transforms.
 */
export declare function withRowCoder<T extends Object>(exemplar: T): PTransform<PCollection<T>, PCollection<T>>;
/**
 * **Note**: Users should not be using `GroupByKey` transforms directly. Use instead
 * `GroupBy`, and `Combine` transforms.
 *
 * `GroupByKey` is the primitive transform in Beam to force *shuffling* of data,
 * which helps us group data of the same key together. It's a necessary primitive
 * for any Beam SDK.
 *
 * `GroupByKey` operations are used under the hood to execute combines,
 * streaming triggers, stateful transforms, etc.
 */
export declare function groupByKey<K, V>(): PTransform<PCollection<KV<K, V>>, PCollection<KV<K, Iterable<V>>>>;
export declare namespace groupByKey {
    var urn: string;
}
/**
 * This transform is used to perform aggregations over groups of elements.
 *
 * It receives a `CombineFn`, which defines functions to create an intermediate
 * aggregator, add elements to it, and transform the aggregator into the expected
 * output.
 *
 * Combines are a valuable transform because they allow for optimizations that
 * can reduce the amount of data being exchanged between workers
 * (a.k.a. "shuffled"). They do this by performing partial aggregations
 * before a `GroupByKey` and after the `GroupByKey`. The partial aggregations
 * help reduce the original data into a single aggregator per key per worker.
 */
export declare function combinePerKey<K, InputT, AccT, OutputT>(combineFn: CombineFn<InputT, AccT, OutputT>): PTransform<PCollection<KV<K, InputT>>, PCollection<KV<K, OutputT>>>;
export declare namespace combinePerKey {
    var urn: string;
}
