import { PCollection } from "../pvalue";
import { PTransform } from "./transform";
import { PaneInfo, Instant, Window, WindowedValue } from "../values";
/**
 * The interface used to apply an elementwise MappingFn to a PCollection.
 *
 * For simple transformations, `PCollection.map` or `PCollection.flatMap`
 * may be simpler to use.
 *
 * See also https://beam.apache.org/documentation/programming-guide/#pardo
 */
export interface DoFn<InputT, OutputT, ContextT = undefined> {
    /**
     * If provided, the default name to use for this operation.
     */
    beamName?: string;
    /**
     * Process a single element from the PCollection, returning an iterable
     * of zero or more result elements.
     *
     * Also takes as input an optional context element which has the same
     * type as was passed into the parDo at construction time (but which is
     * now "activated" in the sense that side inputs, metrics, etc. are
     * available with runtime values/effects).
     */
    process: (element: InputT, context: ContextT) => Iterable<OutputT> | void;
    /**
     * Called once at the start of every bundle, before any `process()` calls.
     *
     * This can be used to amortize any expensive initialization.
     */
    startBundle?: (context: ContextT) => void;
    /**
     * Called once at the end of every bundle, after any `process()` calls.
     *
     * This can be used to clean up expensive initialization and/or flush any
     * elements that were buffered.
     */
    finishBundle?: (context: ContextT) => Iterable<WindowedValue<OutputT>> | void;
}
/**
 * Creates a PTransform that applies a `DoFn` to a PCollection.
 */
export declare function parDo<InputT, OutputT, ContextT extends Object | undefined = undefined>(doFn: DoFn<InputT, OutputT, ContextT>, context?: ContextT): PTransform<PCollection<InputT>, PCollection<OutputT>>;
export declare namespace parDo {
    var urn: string;
}
export declare type SplitOptions = {
    knownTags?: string[];
    unknownTagBehavior?: "error" | "ignore" | "rename" | undefined;
    unknownTagName?: string;
    exclusive?: boolean;
};
/**
 * Splits a single PCollection of objects, with keys k, into an object of
 * PCollections, with the same keys k, where each PCollection consists of the
 * values associated with that key. That is,
 *
 * `PCollection<{a: T, b: U, ...}>` maps to `{a: PCollection<T>, b: PCollection<U>, ...}`
 */
export declare function split<X extends {
    [key: string]: unknown;
}>(tags: string[], options?: SplitOptions): PTransform<PCollection<X>, {
    [P in keyof X]: PCollection<X[P]>;
}>;
export declare function partition<T>(partitionFn: (element: T, numPartitions: number) => number, numPartitions: number): PTransform<PCollection<T>, PCollection<T>[]>;
/**
 * Used to declare the need for parameters such as counters, windowing context,
 * state, etc. that do not have to be provided externally (such as side inputs).
 *
 * This can be useful to bind the context of a parallel operation outside of
 * its application (such as map or pardo).
 */
export declare function withContext<ContextT, T extends DoFn<unknown, unknown, ContextT> | ((input: unknown, context: ContextT) => unknown)>(fn: T, contextSpec: ContextT): T;
/** @internal */
export declare function extractContext(fn: any): any;
/**
 * This is the root class of special parameters that can be provided in the
 * context of a map or DoFn.process method.
 */
export declare class ParDoParam {
    readonly parDoParamName: string;
    /** @internal */
    protected provider: ParamProvider | undefined;
    /** @internal */
    constructor(parDoParamName: string);
}
/**
 * At runtime, one can invoke the special `lookup` method to retrieve the
 * relevant value associated with the currently-being-processed element.
 */
export declare class ParDoLookupParam<T> extends ParDoParam {
    lookup(): T;
}
/**
 * At runtime, one can invoke the special `update` method to update the
 * relevant value associated with the currently-being-processed element.
 */
export declare class ParDoUpdateParam<T> extends ParDoParam {
    update(value: T): void;
}
/**
 * This is the magic class that wires up the ParDoParams to their values
 * at runtime.
 *
 * @internal
 */
export interface ParamProvider {
    lookup<T>(param: ParDoLookupParam<T>): T;
    update<T>(param: ParDoUpdateParam<T>, value: T): void;
}
export declare function windowParam(): ParDoLookupParam<Window>;
export declare function timestampParam(): ParDoLookupParam<Instant>;
export declare function paneInfoParam(): ParDoLookupParam<PaneInfo>;
interface SideInputAccessor<PCollT, AccessorT, ValueT> {
    accessPattern: string;
    toValue: (AccessorT: any) => ValueT;
}
/**
 * Used to access side inputs corresponding to a given element from within a
 * `process()` method.
 *
 * See also https://beam.apache.org/documentation/programming-guide/#side-inputs
 */
export declare class SideInputParam<PCollT, AccessorT, ValueT> extends ParDoLookupParam<ValueT> {
    pcoll: PCollection<PCollT>;
    accessor: SideInputAccessor<PCollT, AccessorT, ValueT>;
    constructor(pcoll: PCollection<PCollT>, accessor: SideInputAccessor<PCollT, AccessorT, ValueT>);
    sideInputId: string;
}
export declare function iterableSideInput<T>(pcoll: PCollection<T>): SideInputParam<T, Iterable<T>, Iterable<T>>;
export declare function singletonSideInput<T>(pcoll: PCollection<T>, defaultValue?: T | undefined): SideInputParam<T, Iterable<T>, T>;
/**
 * The superclass of all metric accessors, such as counters and distributions.
 */
export declare class Metric<T> extends ParDoUpdateParam<T> {
    readonly metricType: string;
    readonly name: string;
    constructor(metricType: string, name: string);
}
export declare class Counter extends Metric<number> {
    constructor(name: string);
    increment(value?: number): void;
}
export declare function counter(name: string): Counter;
export declare function distribution(name: string): Metric<number>;
export {};
